@@ -4,6 +4,10 @@
 //! manual](http://difftastic.wilfred.me.uk/).
 //!
 
+// This tends to trigger on larger tuples of simple types, and naming
+// them would probably be worse for readability.
+#![allow(clippy::type_complexity)]
+
 mod context;
 mod dijkstra;
 mod files;
@@ -15,6 +19,7 @@ mod line_parser;
 mod lines;
 mod positions;
 mod side_by_side;
+mod sliders;
 mod style;
 mod summary;
 mod syntax;
@@ -24,6 +29,8 @@ mod tree_sitter_parser;
 extern crate log;
 
 use crate::hunks::{matched_pos_to_hunks, merge_adjacent};
+use context::opposite_positions;
+use files::read_files_or_die;
 use guess_language::guess;
 use log::info;
 use mimalloc::MiMalloc;
@@ -37,6 +44,7 @@ static GLOBAL: MiMalloc = MiMalloc;
 
 use atty::Stream;
 use clap::{crate_version, App, AppSettings, Arg};
+use sliders::fix_all_sliders;
 use std::{env, path::Path};
 use summary::DiffResult;
 use typed_arena::Arena;
@@ -45,6 +53,7 @@ use walkdir::WalkDir;
 use crate::{
     dijkstra::mark_syntax,
     files::{is_probably_binary, read_or_die},
+    line_parser as lp,
     lines::MaxLine,
     syntax::{change_positions, init_info},
     tree_sitter_parser as tsp,
@@ -240,12 +249,12 @@ fn main() {
 
 /// Print a diff between two files.
 fn diff_file(display_path: &str, lhs_path: &Path, rhs_path: &Path) -> DiffResult {
-    let lhs_bytes = read_or_die(lhs_path);
-    let rhs_bytes = read_or_die(rhs_path);
+    let (lhs_bytes, rhs_bytes) = read_files_or_die(lhs_path, rhs_path);
+    diff_file_content(display_path, &lhs_bytes, &rhs_bytes)
+}
 
-    let lhs_binary = is_probably_binary(&lhs_bytes);
-    let rhs_binary = is_probably_binary(&rhs_bytes);
-    if lhs_binary || rhs_binary {
+fn diff_file_content(display_path: &str, lhs_bytes: &[u8], rhs_bytes: &[u8]) -> DiffResult {
+    if is_probably_binary(lhs_bytes) || is_probably_binary(rhs_bytes) {
         return DiffResult {
             path: display_path.into(),
             language: None,
@@ -258,10 +267,10 @@ fn diff_file(display_path: &str, lhs_path: &Path, rhs_path: &Path) -> DiffResult
     }
 
     // TODO: don't replace tab characters inside string literals.
-    let lhs_src = String::from_utf8_lossy(&lhs_bytes)
+    let lhs_src = String::from_utf8_lossy(lhs_bytes)
         .to_string()
         .replace("\t", "    ");
-    let rhs_src = String::from_utf8_lossy(&rhs_bytes)
+    let rhs_src = String::from_utf8_lossy(rhs_bytes)
         .to_string()
         .replace("\t", "    ");
 
@@ -277,26 +286,41 @@ fn diff_file(display_path: &str, lhs_path: &Path, rhs_path: &Path) -> DiffResult
     };
     let ts_lang = guess(path, guess_src).map(tsp::from_language);
 
-    let arena = Arena::new();
-    let (lang_name, lhs, rhs) = match ts_lang {
-        Some(ts_lang) => (
-            Some(ts_lang.name.into()),
-            tsp::parse(&arena, &lhs_src, &ts_lang),
-            tsp::parse(&arena, &rhs_src, &ts_lang),
-        ),
-        None => (
-            None,
-            line_parser::parse(&arena, &lhs_src),
-            line_parser::parse(&arena, &rhs_src),
-        ),
+    if lhs_bytes == rhs_bytes {
+        return DiffResult {
+            path: display_path.into(),
+            language: ts_lang.map(|l| l.name.into()),
+            binary: true,
+            lhs_src: "".into(),
+            rhs_src: "".into(),
+            lhs_positions: vec![],
+            rhs_positions: vec![],
+        };
+    }
+
+    let (lang_name, lhs_positions, rhs_positions) = match ts_lang {
+        Some(ts_lang) => {
+            let arena = Arena::new();
+            let lhs = tsp::parse(&arena, &lhs_src, &ts_lang);
+            let rhs = tsp::parse(&arena, &rhs_src, &ts_lang);
+
+            init_info(&lhs, &rhs);
+            mark_syntax(lhs.get(0).copied(), rhs.get(0).copied());
+
+            fix_all_sliders(&lhs);
+            fix_all_sliders(&rhs);
+
+            let lhs_positions = change_positions(&lhs_src, &rhs_src, &lhs);
+            let rhs_positions = change_positions(&rhs_src, &lhs_src, &rhs);
+            (Some(ts_lang.name.into()), lhs_positions, rhs_positions)
+        }
+        None => {
+            let lhs_positions = lp::change_positions(&lhs_src, &rhs_src);
+            let rhs_positions = lp::change_positions(&rhs_src, &lhs_src);
+            (None, lhs_positions, rhs_positions)
+        }
     };
 
-    init_info(&lhs, &rhs);
-    mark_syntax(lhs.get(0).copied(), rhs.get(0).copied());
-
-    let lhs_positions = change_positions(&lhs_src, &rhs_src, &lhs);
-    let rhs_positions = change_positions(&rhs_src, &lhs_src, &rhs);
-
     DiffResult {
         path: display_path.into(),
         language: lang_name,
@@ -326,22 +350,25 @@ fn diff_directories(lhs_dir: &Path, rhs_dir: &Path) -> Vec<DiffResult> {
         let rel_path = lhs_path.strip_prefix(lhs_dir).unwrap();
         let rhs_path = Path::new(rhs_dir).join(rel_path);
 
-        res.push(diff_file(&rel_path.to_string_lossy(), &lhs_path, &rhs_path));
+        res.push(diff_file(&rel_path.to_string_lossy(), lhs_path, &rhs_path));
     }
     res
 }
 
 fn print_diff_result(summary: &DiffResult) {
     if summary.binary {
-        print!("{}", style::header(&summary.path, 1, 1, "binary"));
+        println!("{}", style::header(&summary.path, 1, 1, "binary"));
         return;
     }
 
+    let opposite_to_lhs = opposite_positions(&summary.lhs_positions);
+    let opposite_to_rhs = opposite_positions(&summary.rhs_positions);
+
     let hunks = matched_pos_to_hunks(&summary.lhs_positions, &summary.rhs_positions);
     let hunks = merge_adjacent(
         &hunks,
-        &summary.lhs_positions,
-        &summary.rhs_positions,
+        &opposite_to_lhs,
+        &opposite_to_rhs,
         summary.lhs_src.max_line(),
         summary.rhs_src.max_line(),
     );
@@ -385,3 +412,17 @@ fn print_diff_result(summary: &DiffResult) {
         );
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn test_diff_identical_content() {
+        let s = "foo";
+        let res = diff_file_content("foo.el", s.as_bytes(), s.as_bytes());
+
+        assert_eq!(res.lhs_positions, vec![]);
+        assert_eq!(res.rhs_positions, vec![]);
+    }
+}
