@@ -42,13 +42,29 @@ impl<'a> fmt::Debug for ChangeKind<'a> {
 
 /// Fields that are common to both `Syntax::List` and `Syntax::Atom`.
 pub struct SyntaxInfo<'a> {
+    /// The syntax node that occurs after this one, in a depth-first
+    /// tree traversal.
     next: Cell<Option<&'a Syntax<'a>>>,
+    /// The syntax node that occurs before this one, in a depth-first
+    /// tree traversal.
     prev: Cell<Option<&'a Syntax<'a>>>,
+    /// The parent syntax node, if present.
     parent: Cell<Option<&'a Syntax<'a>>>,
+    /// Does the previous syntax node occur on the same line as the
+    /// first line of this node?
     prev_is_contiguous: Cell<bool>,
-    change: Cell<Option<ChangeKind<'a>>>,
+    /// Whether or not this syntax node has changed. This value is set
+    /// when computing the diff with another syntax tree.
+    pub change: Cell<Option<ChangeKind<'a>>>,
+    /// The number of nodes that are ancestors of this one.
     num_ancestors: Cell<u32>,
+    /// A number that uniquely identifies this syntax node.
     unique_id: Cell<u32>,
+    /// A number that uniquely identifies the content of this syntax
+    /// node. This may be the same as nodes on the other side of the
+    /// diff, or nodes at different positions.
+    ///
+    /// Values are sequential, not hashes. Collisions never occur.
     content_id: Cell<u32>,
 }
 
@@ -180,6 +196,19 @@ impl<'a> Syntax<'a> {
         close_content: &str,
         close_position: Vec<SingleLineSpan>,
     ) -> &'a Syntax<'a> {
+        // Don't bother creating a list if we have no open/close and
+        // there's only one child. This occurs in small files with
+        // thorough tree-sitter parsers: you get parse trees like:
+        //
+        // (compilation-unit (top-level-def (function ...)))
+        //
+        // This is a small performance win as it makes the difftastic
+        // syntax tree smaller. It also really helps when looking at
+        // debug output for small inputs.
+        if children.len() == 1 && open_content.is_empty() && close_content.is_empty() {
+            return children[0];
+        }
+
         let mut num_descendants = 0;
         for child in &children {
             num_descendants += match child {
@@ -238,7 +267,7 @@ impl<'a> Syntax<'a> {
     /// A content ID of this syntax node. Two nodes have the same
     /// content ID if they have the same content, regardless of
     /// position.
-    fn content_id(&self) -> u32 {
+    pub fn content_id(&self) -> u32 {
         self.info().content_id.get()
     }
 
@@ -262,6 +291,10 @@ impl<'a> Syntax<'a> {
         position.last().map(|lp| lp.line)
     }
 
+    pub fn change(&'a self) -> Option<ChangeKind<'a>> {
+        self.info().change.get()
+    }
+
     pub fn set_change(&self, ck: ChangeKind<'a>) {
         self.info().change.set(Some(ck));
     }
@@ -308,7 +341,7 @@ type ContentKey = (
     Option<AtomKind>,
 );
 
-fn set_content_id<'a>(nodes: &[&'a Syntax<'a>], existing: &mut HashMap<ContentKey, u32>) {
+fn set_content_id(nodes: &[&Syntax], existing: &mut HashMap<ContentKey, u32>) {
     for node in nodes {
         let key: ContentKey = match node {
             List {
@@ -489,11 +522,8 @@ pub enum TokenKind {
 pub enum MatchKind {
     Unchanged {
         highlight: TokenKind,
-        self_pos: (Vec<SingleLineSpan>, Vec<SingleLineSpan>),
-        // If it's a matched atom, only use the first vec. If it'a
-        // matched delimiter pair, use both vecs.
-        // TODO: better data type.
-        opposite_pos: (Vec<SingleLineSpan>, Vec<SingleLineSpan>),
+        self_pos: Vec<SingleLineSpan>,
+        opposite_pos: Vec<SingleLineSpan>,
     },
     Novel {
         highlight: TokenKind,
@@ -508,7 +538,7 @@ pub enum MatchKind {
 impl MatchKind {
     pub fn first_opposite_span(&self) -> Option<SingleLineSpan> {
         match self {
-            MatchKind::Unchanged { opposite_pos, .. } => opposite_pos.0.first().copied(),
+            MatchKind::Unchanged { opposite_pos, .. } => opposite_pos.first().copied(),
             MatchKind::UnchangedCommentPart { opposite_pos, .. } => opposite_pos.first().copied(),
             MatchKind::Novel { .. } => None,
             MatchKind::ChangedCommentPart {} => None,
@@ -530,7 +560,7 @@ pub struct MatchedPos {
 }
 
 // "foo bar" -> vec!["foo", " ", "bar"]
-fn split_words(s: &str) -> Vec<String> {
+pub fn split_words(s: &str) -> Vec<String> {
     lazy_static! {
         static ref RE: Regex = Regex::new(r"[a-zA-Z0-9]+|\n|[^a-zA-Z0-9\n]").unwrap();
     }
@@ -604,7 +634,8 @@ impl MatchedPos {
     fn new(
         ck: ChangeKind,
         highlight: TokenKind,
-        pos: (&[SingleLineSpan], &[SingleLineSpan]),
+        pos: &[SingleLineSpan],
+        is_close: bool,
     ) -> Vec<Self> {
         let kind = match ck {
             ReplacedComment(this, opposite) => {
@@ -621,8 +652,8 @@ impl MatchedPos {
 
                 return split_comment_words(
                     this_content,
-                    // TODO: handle the whole pos.0 here.
-                    pos.0[0],
+                    // TODO: handle the whole pos here.
+                    pos[0],
                     opposite_content,
                     opposite_pos[0],
                 );
@@ -633,13 +664,19 @@ impl MatchedPos {
                         open_position,
                         close_position,
                         ..
-                    } => (open_position.clone(), close_position.clone()),
-                    Atom { position, .. } => (position.clone(), position.clone()),
+                    } => {
+                        if is_close {
+                            close_position.clone()
+                        } else {
+                            open_position.clone()
+                        }
+                    }
+                    Atom { position, .. } => position.clone(),
                 };
 
                 MatchKind::Unchanged {
                     highlight,
-                    self_pos: (pos.0.to_vec(), pos.1.to_vec()),
+                    self_pos: pos.to_vec(),
                     opposite_pos,
                 }
             }
@@ -649,7 +686,7 @@ impl MatchedPos {
         // Create a MatchedPos for every line that `pos` covers.
         // TODO: what about opposite pos?
         let mut res = vec![];
-        for line_pos in pos.0 {
+        for line_pos in pos {
             // Don't create a MatchedPos for empty positions. This
             // occurs when we have lists with empty open/close
             // delimiter positions, such as the top-level list of syntax items.
@@ -671,7 +708,7 @@ impl MatchedPos {
 pub fn change_positions<'a>(
     src: &str,
     opposite_src: &str,
-    nodes: &[&Syntax<'a>],
+    nodes: &[&'a Syntax<'a>],
 ) -> Vec<MatchedPos> {
     let nl_pos = NewlinePositions::from(src);
     let opposite_nl_pos = NewlinePositions::from(opposite_src);
@@ -684,27 +721,26 @@ pub fn change_positions<'a>(
 fn change_positions_<'a>(
     nl_pos: &NewlinePositions,
     opposite_nl_pos: &NewlinePositions,
-    nodes: &[&Syntax<'a>],
+    nodes: &[&'a Syntax<'a>],
     positions: &mut Vec<MatchedPos>,
 ) {
     for node in nodes {
+        let change = node
+            .change()
+            .unwrap_or_else(|| panic!("Should have changes set in all nodes: {:#?}", node));
+
         match node {
             List {
-                info,
                 open_position,
                 children,
                 close_position,
                 ..
             } => {
-                let change = info
-                    .change
-                    .get()
-                    .unwrap_or_else(|| panic!("Should have changes set in all nodes: {:#?}", node));
-
                 positions.extend(MatchedPos::new(
                     change,
                     TokenKind::Delimiter,
-                    (open_position, close_position),
+                    open_position,
+                    false,
                 ));
 
                 change_positions_(nl_pos, opposite_nl_pos, children, positions);
@@ -712,25 +748,16 @@ fn change_positions_<'a>(
                 positions.extend(MatchedPos::new(
                     change,
                     TokenKind::Delimiter,
-                    // TODO: use open position here too (currently
-                    // breaks display).
-                    (close_position, close_position),
+                    close_position,
+                    true,
                 ));
             }
-            Atom {
-                info,
-                position,
-                kind,
-                ..
-            } => {
-                let change = info
-                    .change
-                    .get()
-                    .unwrap_or_else(|| panic!("Should have changes set in all nodes: {:#?}", node));
+            Atom { position, kind, .. } => {
                 positions.extend(MatchedPos::new(
                     change,
                     TokenKind::Atom(*kind),
-                    (position, &[]),
+                    position,
+                    false,
                 ));
             }
         }
@@ -791,12 +818,6 @@ mod tests {
     use super::*;
     use pretty_assertions::assert_eq;
 
-    impl<'a> Syntax<'a> {
-        pub fn change(&'a self) -> Option<ChangeKind<'a>> {
-            self.info().change.get()
-        }
-    }
-
     #[test]
     fn test_comment_and_atom_differ() {
         let pos = vec![SingleLineSpan {
@@ -814,6 +835,22 @@ mod tests {
         assert_ne!(comment, atom);
     }
 
+    #[test]
+    fn test_flatten_trivial_list() {
+        let pos = vec![SingleLineSpan {
+            line: 0.into(),
+            start_col: 2,
+            end_col: 3,
+        }];
+
+        let arena = Arena::new();
+        let atom = Syntax::new_atom(&arena, pos, "foo", AtomKind::Normal);
+
+        let trivial_list = Syntax::new_list(&arena, "", vec![], vec![atom], "", vec![]);
+
+        assert!(matches!(trivial_list, Atom { .. }));
+    }
+
     #[test]
     fn test_multiline_comment_ignores_leading_whitespace() {
         let pos = vec![SingleLineSpan {
